<?php

namespace Ardent\Game;

use Ardent\Game\dto\Color;

class Noise
{
    private readonly PerlinNoise $noise;

    public function __construct(
        ?int $seed = null,
    )
    {
        $this->noise = new PerlinNoise($seed);
    }

    public function generate(int $x, int $y): Color
    {
        $deepwater = new Color(0, 110, 180);
        $water = new Color(0, 156, 244);
        $sand = new Color(238, 226, 104);
        $grass = new Color(25, 171, 64);
        $mountain = new Color(96, 93, 100);
        $snow = new Color(255, 255, 255);

        $rand = $this->noise->random2D($x, $y, 150);

        $transitions = [
            [-1.0, $deepwater],
            [-0.9, $water],
            [0.0, $sand],
            [0.8, $grass],
            [0.997, $mountain],
            [1.0, $snow],
        ];

        for ($i = 0; $i < count($transitions) - 1; $i++) {
            $transition = $transitions[$i];
            $nextTransition = $transitions[$i + 1];
            if ($rand >= $transition[0] && $rand < $nextTransition[0]) {
                return $transition[1]->lerp($nextTransition[1], ($rand - $transition[0]) * (1.0 / ($nextTransition[0] - $transition[0])));
            }
        }
    }
}